"""Library with compatible bulb types."""


class Features:
    """Defines the supported features."""

    color: bool
    color_tmp: bool
    effect: bool
    brightness: bool

    def __init__(
        self, color: bool, color_tmp: bool, effect: bool, brightness: bool
    ) -> None:
        """Init the features with type."""
        self.color = color
        self.color_tmp = color_tmp
        self.effect = effect
        self.brightness = brightness


class KelvinRange:
    """Deines the kelvin range."""

    max: int
    min: int

    def __init__(self, max: int, min: int) -> None:
        """Init for the kelvin range class."""
        self.max = max
        self.min = min


class BulbType:
    """BulbType object to define functions and features of the bulb."""

    features: Features
    name: str
    kelvin_range: KelvinRange

    def __init__(
        self, features: Features, name: str, kelvin_range: KelvinRange
    ) -> None:
        """Create a bulb object with different features."""
        self.features = features
        self.name = name
        self.kelvin_range = kelvin_range


class BulbLib:
    """Provides all existing bulbs."""

    BULBLIST = [
        # Only Brightness
        BulbType(
            name="ESP01_SHDW_01",
            features=Features(
                brightness=True, color=False, effect=False, color_tmp=False
            ),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        BulbType(
            name="ESP01_SHDW_01",
            features=Features(
                brightness=True, color=False, effect=False, color_tmp=False
            ),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        # Brightness and Effects
        BulbType(
            name="ESP06_SHDW9_01",
            features=Features(
                brightness=True, color=False, effect=True, color_tmp=False
            ),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        BulbType(
            name="ESP06_SHDW1_01",
            features=Features(
                brightness=True, color=False, effect=True, color_tmp=False
            ),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        # Brightness and Color Temp
        BulbType(
            name="ESP01_SHTW1C_31",
            features=Features(
                brightness=True, color=False, effect=False, color_tmp=True
            ),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        BulbType(
            name="ESP17_SHTW9_01",
            features=Features(
                brightness=True, color=False, effect=False, color_tmp=True
            ),
            kelvin_range=KelvinRange(min=2000, max=5000),
        ),
        # Brightness, Color Temp and Effect
        BulbType(
            name="ESP56_SHTW3_01",
            features=Features(
                brightness=True, color=False, effect=True, color_tmp=True
            ),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        BulbType(
            name="ESP15_SHTW1_01I",
            features=Features(
                brightness=True, color=False, effect=True, color_tmp=True
            ),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        BulbType(
            name="ESP03_SHTW1C_01",
            features=Features(
                brightness=True, color=False, effect=True, color_tmp=True
            ),
            kelvin_range=KelvinRange(min=2700, max=6500),
        ),
        BulbType(
            name="ESP03_SHTW1W_01",
            features=Features(
                brightness=True, color=False, effect=True, color_tmp=True
            ),
            kelvin_range=KelvinRange(min=2700, max=6500),
        ),
        # Brightness, Color Temp, Color and Effect / all features
        BulbType(
            name="ESP01_SHRGB1C_31",
            features=Features(brightness=True, color=True, effect=True, color_tmp=True),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        # Test device
        BulbType(
            name="ESP01_SHRGB_03",
            features=Features(brightness=True, color=True, effect=True, color_tmp=True),
            kelvin_range=KelvinRange(min=2700, max=6500),
        ),
        BulbType(
            name="ESP03_SHRGBP_31",
            features=Features(brightness=True, color=True, effect=True, color_tmp=True),
            kelvin_range=KelvinRange(min=2000, max=6500),
        ),
        BulbType(
            name="ESP03_SHRGB1C_01",
            features=Features(brightness=True, color=True, effect=True, color_tmp=True),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        BulbType(
            name="ESP03_SHRGB1W_01",
            features=Features(brightness=True, color=True, effect=True, color_tmp=True),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
        BulbType(
            name="ESP03_SHRGB3_01ABI",
            features=Features(brightness=True, color=True, effect=True, color_tmp=True),
            kelvin_range=KelvinRange(min=2200, max=6500),
        ),
    ]

    def getBulbList(self) -> list:
        """Retrun the list of known bulbs as BulbType list."""
        return self.BULBLIST



"""Start up a fake bulb to test features without a real bulb."""
import json
import socketserver
import threading

BULB_INIT_PILOT = json.loads(
    '{\
   "method":"getPilot",\
   "env":"pro",\
   "result":{\
      "mac":"ABCABCABCABC",\
      "rssi":-62,\
      "src":"",\
      "state":false,\
      "sceneId":0,\
      "r":255,\
      "g":127,\
      "b":0,\
      "c":0,\
      "w":0,\
      "temp":0,\
      "dimming":13\
   }\
}\
'
)

BULB_SYS_CONFIG = json.loads(
    '{"method":"getSystemConfig","env":"pro","result":{"mac":"a8bb5006033d","homeId":653906,"roomId":989983,"moduleName"\
    :"","fwVersion":"1.21.0","groupId":0,"drvConf":[20,2],\
    "ewf":[255,0,255,255,0,0,0],"ewfHex":"ff00ffff000000","ping":0}}'
)

BULB_JSON_ERROR = b'{"env":"pro","error":{"code":-32700,"message":"Parse error"}}'


def udp_fake_bulb():
    """Start a fake bulb instance."""

    class BulbUDPRequestHandler(socketserver.DatagramRequestHandler):
        """Class for UDP handler."""

        def handle(self):
            """Handle the request."""
            json_data = None
            data = self.rfile.readline().strip()
            print(f"Reuest:{data}")
            try:
                json_data = json.loads(data.decode())
            except json.JSONDecodeError:
                self.wfile.write(BULB_JSON_ERROR)

            if json_data["method"] == "setPilot":
                return_data = self.setPilot(json_data)
                self.wfile.write(return_data)
            if json_data["method"] == "getPilot":
                print(f"Response:{json.dumps(BULB_INIT_PILOT)}")
                self.wfile.write(bytes(json.dumps(BULB_INIT_PILOT), "utf-8"))
            if json_data["method"] == "getSystemConfig":
                self.wfile.write(bytes(json.dumps(BULB_SYS_CONFIG), "utf-8"))

        def setPilot(self, json_data):
            """Change the values in the state."""
            for name, value in json_data["params"].items():
                BULB_INIT_PILOT["result"][name] = value
                print(f"Change: {json.dumps(BULB_INIT_PILOT)}")
            return b'{"method":"setPilot","env":"pro","result":{"success":true}}'

    UDPServerObject = socketserver.ThreadingUDPServer(
        server_address=("127.0.0.1", 38899), RequestHandlerClass=BulbUDPRequestHandler
    )
    UDPServerObject.daemon_threads = True
    UDPServerObject.serve_forever()


def startup_bulb(module_name="ESP01_SHRGB_03"):
    """Startup the bulb."""
    BULB_SYS_CONFIG["result"]["moduleName"] = module_name
    thread = threading.Thread(target=udp_fake_bulb)
    thread.daemon = True
    thread.start()



"""Command-line interface to interact with wizlight devices."""
import asyncio
from functools import wraps

import click

from pywizlight import discovery, wizlight


def coro(f):
    """Allow to use async in click."""

    @wraps(f)
    def wrapper(*args, **kwargs):
        """Async wrapper."""
        return asyncio.run(f(*args, **kwargs))

    return wrapper


@click.group()
@click.version_option()
def main():
    """Command-line tool to interact with Wizlight bulbs."""


@main.command("discover")
@coro
async def discover():
    """Disocver bulb in the local network."""
    click.echo("Waiting for broadcast packages...")

    bulbs = await discovery.find_wizlights(discovery)
    for bulb in bulbs:
        click.echo(bulb.__dict__)


@main.command("on")
@coro
@click.option("--ip", prompt="IP address of the bulb", help="IP address of the bulb.")
async def turn_on(ip):
    """Turn a given bulb on."""
    click.echo("Turning on %s" % ip)
    bulb = wizlight(ip)
    await bulb.turn_on()


@main.command("off")
@coro
@click.option("--ip", prompt="IP address of the bulb", help="IP address of the bulb.")
async def turn_off(ip):
    """Turn a given bulb off."""
    click.echo("Turning off %s" % ip)
    bulb = wizlight(ip)
    await bulb.turn_off()


@main.command("state")
@coro
@click.option("--ip", prompt="IP address of the bulb", help="IP address of the bulb.")
async def state(ip):
    """Get the current state of a given bulb."""
    click.echo("Get the state from %s" % ip)
    bulb = wizlight(ip)
    state = await bulb.updateState()
    click.echo(state.__dict__["pilotResult"])


if __name__ == "__main__":
    main()
